import {createBlock} from '@wordpress/blocks';
import {
	useBlockProps,
	useInnerBlocksProps,
	useBlockEditContext,
	store as blockEditorStore,
	InspectorControls,
	ButtonBlockAppender,
} from '@wordpress/block-editor';
import { PanelBody, RangeControl, CheckboxControl, TextareaControl } from "@wordpress/components";
import { useRefEffect } from '@wordpress/compose';
import { select, subscribe } from '@wordpress/data';
import { Swiper } from 'swiper';
import { A11y, FreeMode, Pagination } from 'swiper/modules';
import { memo, useState } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import "./editor.scss";

const Carousel = memo(({clientId, attributes, innerBlocksProps, isSelected})=> {
  const carouselRef = useRefEffect((el) => {
    const params = {
      ...attributes
    }

    // init carousel
    const initCarousel = (container, params = {}) => {
      const parameters = {
        a11y: {
          scrollOnFocus: true,
          containerMessage: params.ariaLabel
        },
        createElements: true,
        initialSlide: 0,
        centeredSlides: false,
        slideToClickedSlide: true,
        simulateTouch: false,
        modules: [Pagination, A11y, FreeMode],
        slidesPerView: params.perPageMobile,
        freemode: true,
        grabCursor: true,
        breakpoints: {
          768: {
            slidesPerView: params.perPageTablet
          },
          960: {
            slidesPerView: params.perPage
          },
        },
        pagination: {
          enabled: true,
          clickable: true,
          hideOnClick: false
        }
      };
      return new Swiper(container, parameters);
    }

    let carousel = initCarousel(el, params);

    let slideOrder = select( blockEditorStore ).getBlockOrder( clientId );

    const unsubscribeSliderUpdateListener = subscribe(() => {
      const currSlideOrder = select( blockEditorStore ).getBlockOrder( clientId );
      
      // Check if slider has been changed.
      if (currSlideOrder.toString() !== slideOrder.toString()) {
        const selectedBlock = select( blockEditorStore ).getSelectedBlock();
        const slideAdded = currSlideOrder.length > slideOrder.length;
				const slideRemoved = currSlideOrder.length < slideOrder.length;
				const slideMoved = currSlideOrder.length === slideOrder.length;
				const activeIndex = carousel.activeIndex;

        // Store the current slide order before destroying the slider instance.
				slideOrder = currSlideOrder;
        carousel.destroy();

        window.requestAnimationFrame(() => {
          // init carousel
          carousel = initCarousel(el, params);

          // Determine where carousel should go
          let slideToIndex = activeIndex;
          if ( slideAdded ) {
            slideToIndex = slideOrder.length;
          } else if ( slideRemoved ) {
            slideToIndex = activeIndex - 1;
          } else if ( slideMoved ) {
            slideToIndex = slideOrder.findIndex(
              (clientId) => clientId === selectedBlock.clientId // eslint-disable-line no-shadow
            );
          }

          if ( slideToIndex < 0 ) {
						slideToIndex = 0;
					}

          carousel.slideTo(slideToIndex, 0);

        })
      }
    });
    return () => {
      unsubscribeSliderUpdateListener();
      carousel.destroy();
    }
  })

  return (
    <>
      <div className="swiper ucla-carousel" ref={carouselRef}>
        <div {...innerBlocksProps} />
      </div>
      {isSelected &&
        <ButtonBlockAppender
          className="slider-appender has-icon"
          rootClientId={ clientId }
        />
      }
    </>
  )
})

export default function Edit({attributes, setAttributes, isSelected}) {
  let { perPage, perPageTablet, perPageMobile, ariaLabel } = attributes;
  const [ showBreakPoints, setShowBreakPoints ] = useState(perPageTablet || perPageMobile);
  const { clientId } = useBlockEditContext();
  const blockProps = useBlockProps();
  const innerBlocksProps = useInnerBlocksProps(
    {className: 'swiper-wrapper'},
    {
      allowedBlocks: ["ucla-wordpress-plugin/carousel-slide"],
      template: [
        ["ucla-wordpress-plugin/carousel-slide", {}],
        ["ucla-wordpress-plugin/carousel-slide", {}],
      ],
      orientation: 'horizontal',
      renderAppender: false,
    }
  )
  const onResponsiveChange = value => {
    setShowBreakPoints(!showBreakPoints);
    if (value) {
      setAttributes({perPageMobile: null, perPageTablet: null});
    }
  }
  return (
    <>
      <InspectorControls>
      <PanelBody title="Settings">

          <RangeControl
            label="Number of Items per slide"
            max={4}
            min={1}
            step={1}
            value={perPage}
            onChange={(value) => {
              setAttributes({ perPage: value });
            }}
          />
          {showBreakPoints &&
          <>
            <RangeControl
              label="Number of Items per slide on tablet"
              max={4}
              min={1}
              step={1}
              value={perPageTablet}
              onChange={(value) => {
                setAttributes({ perPageTablet: value });
              }}
            />
            <RangeControl
              label="Number of Items per slide on mobile"
              max={4}
              min={1}
              step={1}
              value={perPageMobile}
              onChange={(value) => {
                setAttributes({ perPageMobile: value });
              }}
            />
          </>
          }
          <CheckboxControl
            checked={showBreakPoints}
            onChange={()=>onResponsiveChange(showBreakPoints)}
            label="Responsive Slides"
            help="Change the number of slides on tablet or mobile"
          />
          <hr />
          <TextareaControl
            label="Aria Label"
            help="Describe the purpose of this carousel"
            value={ariaLabel}
            onChange={(value)=>setAttributes({ariaLabel: value})}
          />
        </PanelBody>
      </InspectorControls>
      <div {...blockProps}>
        <Carousel
          clientId={clientId}
          attributes={attributes}
          innerBlocksProps={innerBlocksProps}
          isSelected={isSelected}
        />
      </div>
    </>
  )
}